@@ -32,6 +32,7 @@
 #include <spa/support/system.h>
 #include <spa/pod/parser.h>
 #include <spa/pod/filter.h>
+#include <spa/pod/dynamic.h>
 #include <spa/node/utils.h>
 #include <spa/debug/types.h>
 #include <spa/utils/string.h>
@@ -591,10 +592,10 @@ static const struct pw_resource_events resource_events = {
 };
 
 static int
-global_bind(void *_data, struct pw_impl_client *client, uint32_t permissions,
+global_bind(void *object, struct pw_impl_client *client, uint32_t permissions,
 	    uint32_t version, uint32_t id)
 {
-	struct pw_impl_node *this = _data;
+	struct pw_impl_node *this = object;
 	struct pw_global *global = this->global;
 	struct pw_resource *resource;
 	struct resource_data *data;
@@ -841,6 +842,7 @@ int pw_impl_node_set_driver(struct pw_impl_node *node, struct pw_impl_node *driv
 			old->name, old->info.id, driver->name, driver->info.id);
 
 	node->driver_node = driver;
+	node->moved = true;
 
 	pw_loop_invoke(node->data_loop,
 		       do_move_nodes, SPA_ID_INVALID, &driver, sizeof(struct pw_impl_node *),
@@ -857,6 +859,7 @@ static void check_properties(struct pw_impl_node *node)
 	struct pw_context *context = node->context;
 	const char *str, *recalc_reason = NULL;
 	struct spa_fraction frac;
+	uint32_t value;
 	bool driver;
 
 	if ((str = pw_properties_get(node->properties, PW_KEY_PRIORITY_DRIVER))) {
@@ -935,6 +938,15 @@ static void check_properties(struct pw_impl_node *node)
 	}
 	node->lock_quantum = pw_properties_get_bool(node->properties, PW_KEY_NODE_LOCK_QUANTUM, false);
 
+	if ((str = pw_properties_get(node->properties, PW_KEY_NODE_FORCE_QUANTUM))) {
+		if (spa_atou32(str, &value, 0) &&
+		    node->force_quantum != value) {
+		        node->force_quantum = value;
+			node->stamp = ++context->stamp;
+			recalc_reason = "force quantum changed";
+		}
+	}
+
 	if ((str = pw_properties_get(node->properties, PW_KEY_NODE_RATE))) {
                 if (sscanf(str, "%u/%u", &frac.num, &frac.denom) == 2 && frac.denom != 0) {
 			if (node->rate.num != frac.num || node->rate.denom != frac.denom) {
@@ -948,6 +960,15 @@ static void check_properties(struct pw_impl_node *node)
 	}
 	node->lock_rate = pw_properties_get_bool(node->properties, PW_KEY_NODE_LOCK_RATE, false);
 
+	if ((str = pw_properties_get(node->properties, PW_KEY_NODE_FORCE_RATE))) {
+		if (spa_atou32(str, &value, 0) &&
+		    node->force_rate != value) {
+			node->force_rate = value;
+			node->stamp = ++context->stamp;
+			recalc_reason = "force rate changed";
+		}
+	}
+
 	pw_log_debug("%p: driver:%d recalc:%s active:%d", node, node->driver,
 			recalc_reason, node->active);
 
@@ -1026,7 +1047,7 @@ static inline int resume_node(struct pw_impl_node *this, int status)
 		if (pw_node_activation_state_dec(state, 1)) {
 			a->status = PW_NODE_ACTIVATION_TRIGGERED;
 			a->signal_time = nsec;
-			t->signal(t->data);
+			t->signal_func(t->data);
 		}
 	}
 	return 0;
@@ -1124,7 +1145,7 @@ static void node_on_fd_events(struct spa_source *source)
 				this->name, this->info.id, cmd - 1);
 
 		pw_log_trace_fp("%p: got process", this);
-		this->rt.target.signal(this->rt.target.data);
+		this->rt.target.signal_func(this->rt.target.data);
 	}
 }
 
@@ -1217,10 +1238,6 @@ struct pw_impl_node *pw_context_create_node(struct pw_context *context,
 	}
 
 	impl->work = pw_context_get_work_queue(this->context);
-	if (impl->work == NULL) {
-		res = -errno;
-		goto error_clean;
-	}
 	impl->pending_id = SPA_ID_INVALID;
 
 	this->data_loop = context->data_loop;
@@ -1245,9 +1262,9 @@ struct pw_impl_node *pw_context_create_node(struct pw_context *context,
 	this->rt.activation = this->activation->map->ptr;
 	this->rt.target.activation = this->rt.activation;
 	this->rt.target.node = this;
-	this->rt.target.signal = process_node;
+	this->rt.target.signal_func = process_node;
 	this->rt.target.data = this;
-	this->rt.driver_target.signal = process_node;
+	this->rt.driver_target.signal_func = process_node;
 
 	reset_position(this, &this->rt.activation->position);
 	this->rt.activation->sync_timeout = DEFAULT_SYNC_TIMEOUT;
@@ -1596,7 +1613,7 @@ static int node_ready(void *data, int status)
 						state->pending, state->required);
 				dump_states(node);
 			}
-			node->rt.target.signal(node->rt.target.data);
+			node->rt.target.signal_func(node->rt.target.data);
 		}
 
 		if (node->current_pending) {
@@ -1787,6 +1804,7 @@ void pw_impl_node_destroy(struct pw_impl_node *node)
 	struct impl *impl = SPA_CONTAINER_OF(node, struct impl, this);
 	struct pw_impl_port *port;
 	struct pw_impl_node *follower;
+	struct pw_context *context = node->context;
 	bool active, had_driver;
 
 	active = node->active;
@@ -1836,7 +1854,7 @@ void pw_impl_node_destroy(struct pw_impl_node *node)
 	}
 
 	if (active || had_driver)
-		pw_context_recalc_graph(node->context,
+		pw_context_recalc_graph(context,
 				"active node destroy");
 
 	pw_log_debug("%p: free", node);
@@ -1858,7 +1876,7 @@ void pw_impl_node_destroy(struct pw_impl_node *node)
 
 	clear_info(node);
 
-	spa_system_close(node->context->data_system, node->source.fd);
+	spa_system_close(context->data_system, node->source.fd);
 	free(impl);
 }
 
@@ -1951,7 +1969,7 @@ int pw_impl_node_for_each_param(struct pw_impl_node *node,
 	if (pi->user == 1) {
 		struct pw_param *p;
 		uint8_t buffer[4096];
-		struct spa_pod_builder b = { 0 };
+		struct spa_pod_dynamic_builder b;
 	        struct spa_result_node_params result;
 		uint32_t count = 0;
 
@@ -1966,14 +1984,16 @@ int pw_impl_node_for_each_param(struct pw_impl_node *node,
 			if (result.index < index)
 				continue;
 
-			spa_pod_builder_init(&b, buffer, sizeof(buffer));
-			if (spa_pod_filter(&b, &result.param, p->param, filter) != 0)
-				continue;
+			spa_pod_dynamic_builder_init(&b, buffer, sizeof(buffer), 4096);
 
-			pw_log_debug("%p: %d param %u", node, seq, result.index);
-			result_node_params(&user_data, seq, 0, SPA_RESULT_TYPE_NODE_PARAMS, &result);
+			if (spa_pod_filter(&b.b, &result.param, p->param, filter) == 0)  {
+				pw_log_debug("%p: %d param %u", node, seq, result.index);
+				result_node_params(&user_data, seq, 0, SPA_RESULT_TYPE_NODE_PARAMS, &result);
+				count++;
+			}
+			spa_pod_dynamic_builder_clean(&b);
 
-			if (++count == max)
+			if (count == max)
 				break;
 		}
 		res = 0;
@@ -2194,7 +2214,9 @@ int pw_impl_node_set_active(struct pw_impl_node *node, bool active)
 	bool old = node->active;
 
 	if (old != active) {
-		pw_log_debug("%p: %s", node, active ? "activate" : "deactivate");
+		pw_log_debug("%p: %s registered:%d", node,
+				active ? "activate" : "deactivate",
+				node->registered);
 
 		node->active = active;
 		pw_impl_node_emit_active_changed(node, active);
@@ -2202,6 +2224,8 @@ int pw_impl_node_set_active(struct pw_impl_node *node, bool active)
 		if (node->registered)
 			pw_context_recalc_graph(node->context,
 					active ? "node activate" : "node deactivate");
+		else if (!active && node->exported)
+			pw_loop_invoke(node->data_loop, do_node_remove, 1, NULL, 0, true, node);
 	}
 	return 0;
 }
